@@ -62,7 +62,8 @@ def log_command(port=8825, host='127.0.0.1', chroot=False,
     utils.start_server(pid, FORCE, chroot, chdir, uid, gid, umask, loader)
 
 
-def send_command(port=8825, host='127.0.0.1', debug=1, sender=None, to=None,
+def send_command(port=8825, host='127.0.0.1', username=False, password=False,
+                 ssl=False, starttls=False, debug=1, sender=None, to=None,
                  subject=None, body=None, attach=False):
     """
     Sends an email to someone as a test message.
@@ -70,6 +71,9 @@ def send_command(port=8825, host='127.0.0.1', debug=1, sender=None, to=None,
     
     lamson send -port 8825 -host 127.0.0.1 -debug 1 \\
             -sender EMAIL -to EMAIL -subject STR -body STR -attach False'
+
+    There is also a username, password, and starttls option for those 
+    who need it.
     """
     message = mail.MailResponse(From=sender,
                                   To=to,
@@ -78,7 +82,13 @@ def send_command(port=8825, host='127.0.0.1', debug=1, sender=None, to=None,
     if attach:
         message.attach(attach)
 
-    relay = server.Relay(host, port=port, debug=debug)
+    if username == False:
+        username = None
+    if password == False:
+        password = None
+
+    relay = server.Relay(host, port=port, username=username, password=password,
+                         ssl=ssl, starttls=starttls, debug=debug)
     relay.deliver(message)
 
 
@@ -98,17 +108,6 @@ def sendmail_command(port=8825, host='127.0.0.1', debug=0, TRAILING=None):
 
 
 
-def syncdb_command(create=True):
-    """
-    Creates the database and other stuff for Lamson.
-
-    lamson syncdb -create True
-    """
-    settings = utils.import_settings(False, from_dir=os.getcwd())
-    database = utils.configure_database(settings.database_config)
-    if create:
-        database.create()
-
 
 def start_command(pid='./run/smtp.pid', FORCE=False, chroot=False, chdir=".",
                   boot="config.boot", uid=False, gid=False, umask=False):
@@ -203,8 +202,9 @@ def help_command(**options):
         else:
             args.invalid_command_message(lamson.commands, exit_on_error=True)
     else:
-        print "Lamson help:\n"
-        print "\n".join(args.available_help(lamson.commands))
+        print "Available commands:\n"
+        print ", ".join(args.available_commands(lamson.commands))
+        print "\nUse lamson help -for <command> to find out more."
 
 
 def queue_command(pop=False, get=False, keys=False, remove=False, count=False,
@@ -286,15 +286,15 @@ def routes_command(TRAILING=['config.testing'], path=os.getcwd(), test=""):
 
 
 
-def gen_command(project=None):
+def gen_command(project=None, FORCE=False):
     """
     Generates various useful things for you to get you started.
 
-    lamson gen -project STR
+    lamson gen -project STR -FORCE False
     """
     project = project
 
-    if os.path.exists(project):
+    if os.path.exists(project) and not FORCE:
         print "Project %s exists, delete it first." % project
         sys.exit(1)
         return
@@ -302,7 +302,9 @@ def gen_command(project=None):
     prototype = ZipFile(resource_stream(__name__, 'data/prototype.zip'))
     # looks like the very handy ZipFile.extractall is only in python 2.6
 
-    os.makedirs(project)
+    if not os.path.exists(project):
+        os.makedirs(project)
+
     files = prototype.namelist()
 
     for gen_f in files:
@@ -321,60 +323,6 @@ def gen_command(project=None):
             out.write(prototype.read(gen_f))
             out.close()
 
-def spell_command(TRAILING=None, language="en_US"):
-    """
-    Runs the PyEnchant spell checker on the given file.  Use this to train
-    the spell checker for unit test runs when new words show up.
-
-    lamson spell -langauge en_US -- [files]
-
-    It will run interactively and if you perform an edit on the file it will save
-    your changes back to the file and make a backup named with .bak.
-    """
-    sys.path.append(os.getcwd())
-
-    try:
-        __import__("config.testing", globals(), locals())
-    except ImportError, exc:
-        print "Couldn't import config.testing, assuming you know what you're doing."
-
-    if 'PYENCHANT_LIBRARY_PATH' not in os.environ:
-        print "You don't have 'PYENCHANT_LIBRARY_PATH' set, PyEnchant will fail.  Set it in config/testing.py"
-        print "Looks like you don't know what you're doing.  :-)"
-
-    try:
-        from enchant import DictNotFoundError
-        import enchant.checker 
-        from enchant.checker.CmdLineChecker import CmdLineChecker 
-        from enchant.checker import SpellChecker 
-        from enchant.tokenize import EmailFilter, URLFilter 
-    except ImportError, exc:
-        print "PyEnchant failed to import, you probably have to set 'PYENCHANT_LIBRARY_PATH' in config.testing or in the ENV."
-        print "Yep, you don't know what you're doing.  Read the documentation for help."
-        return False
-
-    try:
-        enchant.Dict(language)
-    except DictNotFoundError, exc:
-        print "PyEnchant couldn't find your dictionary.  Put them in ~/.enchant/myspell."
-        print exc
-        return False
-
-    for check_f in TRAILING:
-        text = open(check_f).read()
-        chkr = SpellChecker(language, filters=[EmailFilter, URLFilter]) 
-        chkr.set_text(text) 
-        cmdln = CmdLineChecker() 
-        cmdln.set_checker(chkr) 
-        cmdln.run()
-        new_text = chkr.get_text()
-        if new_text != text:
-            print "Saving old text to %s.bak and writing changes." % check_f
-            open(check_f + ".bak", "w").write(text)
-            open(check_f, "w").write(new_text)
-
-    return True
-
 
 def web_command(basedir=".", port=8888, host='127.0.0.1'):
     """
@@ -407,7 +355,7 @@ def cleanse_command(input=None, output=None):
     try:
         inbox = mailbox.mbox(input)
     except:
-        inbox = mailbox.Maildir(input)
+        inbox = mailbox.Maildir(input, factory=None)
 
     outbox = mailbox.Maildir(output)
 
@@ -440,4 +388,22 @@ def blast_command(input=None, host='127.0.0.1', port=8823, debug=0):
         relay.deliver(msg)
 
 
+def version_command():
+    """
+    Prints the version of Lamson, the reporitory revision, and the
+    file it came from.
+    """
+
+    from lamson import version
+
+    print "Lamson-Version: ", version.VERSION['version']
+    print "Repository-Revision:", version.VERSION['rev'][0]
+    print "Repository-Hash:", version.VERSION['rev'][1]
+    print "Version-File:", version.__file__
+    print ""
+    print "Lamson is Copyright (C) Zed A. Shaw 2008-2009.  Licensed GPLv3."
+    print "If you didn't get a copy of the LICENSE contact the author at:\n"
+    print "   zedshaw@zedshaw.com"
+    print ""
+    print "Have fun."
 
