@@ -1,50 +1,55 @@
 """
 The majority of the server related things Lamson needs to run, like receivers, 
-relays, databases, and queue processors.
+relays, and queue processors.
 """
 
 import smtplib
 import smtpd
 import asyncore
 import threading
+import socket
 import logging
-from sqlalchemy import create_engine
-from sqlalchemy.orm import sessionmaker
 from lamson import queue, mail, routing
 import time
 import traceback
+from lamson.bounce import PRIMARY_STATUS_CODES, SECONDARY_STATUS_CODES, COMBINED_STATUS_CODES
 
 
-class Database(object):
-    """Manages the database configuration for a Router to give
-    the handlers access to the models.  It really doesn't do much
-    other than setup sqlalchemy and get things going."""
+def undeliverable_message(raw_message, failure_type):
+    """
+    Used universally in this file to shove totally screwed messages
+    into the routing.Router.UNDELIVERABLE_QUEUE (if it's set).
+    """
+    if routing.Router.UNDELIVERABLE_QUEUE:
+        key = routing.Router.UNDELIVERABLE_QUEUE.push(raw_message)
 
-    def __init__(self, metadata, url, log_level=logging.DEBUG):
-        self.metadata = metadata
-        self.url = url
-        self.log_level = log_level
-        logging.getLogger('sqlalchemy.engine').setLevel(self.log_level)
+        logging.error("Failed to deliver message because of %r, put it in "
+                      "undeliverable queue with key %r", failure_type, key)
 
-    def create(self):
-        """Called just to create a database that hasn't been setup
-        yet.  You are responsible for making all the tables and
-        then calling this after metadata is configured."""
-        logging.warning("Database created for URL %s." % self.url)
-        self.metadata.create_all(self.engine)
+class SMTPError(Exception):
+    """
+    You can raise this error when you want to abort with a SMTP error code to
+    the client.  This is really only relevant when you're using the
+    SMTPReceiver and the client understands the error.
 
-    def configure(self):
-        """Takes the url and metadata and fires it up so that you can
-        call self.session() to work with the database."""
-        logging.info("Database configured to use %s URL." % self.url)
-        self.engine = create_engine(self.url)
-        self.sessionmaker = sessionmaker(bind = self.engine)
+    If you give a message than it'll use that, but it'll also produce a
+    consistent error message based on your code.  It uses the errors in
+    lamson.bounce to produce them.
+    """
+    def __init__(self, code, message=None):
+        self.code = code
+        self.message = message or self.error_for_code(code)
 
-    def session(self):
-        """Uses the session maker to create a session you can use
-        in a handler."""
-        return self.sessionmaker()
+        Exception.__init__(self, "%d %s" % (self.code, self.message))
 
+    def error_for_code(self, code):
+        primary, secondary, tertiary = str(code)
+        
+        primary = PRIMARY_STATUS_CODES.get(primary, "")
+        secondary = SECONDARY_STATUS_CODES.get(secondary, "")
+        combined = COMBINED_STATUS_CODES.get(primary + secondary, "")
+
+        return " ".join([primary, secondary, combined]).strip()
 
 
 class Relay(object):
@@ -54,14 +59,38 @@ class Relay(object):
     It supports a few simple operations for sending mail, replying, and can
     log the protocol it uses to stderr if you set debug=1 on __init__.
     """
-    def __init__(self, host='127.0.0.1', port=25, debug=0):
+    def __init__(self, host='127.0.0.1', port=25, username=None, password=None,
+                 ssl=False, starttls=False, debug=0):
         """
         The hostname and port we're connecting to, and the debug level (default to 0).
+        Optional username and password for smtp authentication.
+        If ssl is True smtplib.SMTP_SSL will be used.
+        If starttls is True (and ssl False), smtp connection will be put in TLS mode.
         It does the hard work of delivering messages to the relay host.
         """
         self.hostname = host
         self.port = port
         self.debug = debug
+        self.username = username
+        self.password = password
+        self.ssl = ssl
+        self.starttls = starttls
+
+    def configure_relay(self, hostname):
+        if self.ssl:
+            relay_host = smtplib.SMTP_SSL(hostname, self.port)
+        else:
+            relay_host = smtplib.SMTP(hostname, self.port)
+
+        relay_host.set_debuglevel(self.debug)
+
+        if self.starttls:
+            relay_host.starttls()
+        if self.username and self.password:
+            relay_host.login(self.username, self.password)
+
+        assert relay_host, 'Code error, tell Zed.'
+        return relay_host
 
     def deliver(self, message, To=None, From=None):
         """
@@ -71,15 +100,38 @@ class Relay(object):
         You can pass in an alternate To and From, which will be used in the
         SMTP send lines rather than what's in the message.
         """
-        relay_host = smtplib.SMTP(self.hostname, self.port)
-        relay_host.set_debuglevel(self.debug)
-        relay_host.sendmail(From or message['From'], To or message['To'], str(message))
+        recipient = To or message['To']
+        sender = From or message['From']
+
+        hostname = self.hostname or self.resolve_relay_host(recipient)
+
+        try:
+            relay_host = self.configure_relay(hostname)
+        except socket.error:
+            logging.exception("Failed to connect to host %s:%d" % (hostname, self.port))
+            return
+
+        relay_host.sendmail(sender, recipient, str(message))
         relay_host.quit()
 
+    def resolve_relay_host(self, To):
+        import DNS
+        address, target_host = To.split('@')
+        mx_hosts = DNS.mxlookup(target_host)
+
+        if not mx_hosts:
+            logging.debug("Domain %r does not have an MX record, using %r instead.", target_host, target_host)
+            return target_host
+        else:
+            logging.debug("Delivering to MX record %r for target %r", mx_hosts[0], target_host)
+            return mx_hosts[0][1]
+
+
     def __repr__(self):
         """Used in logging and debugging to indicate where this relay goes."""
         return "<Relay to (%s:%d)>" % (self.hostname, self.port)
 
+
     def reply(self, original, From, Subject, Body):
         """Calls self.send but with the from and to of the original message reversed."""
         self.send(original['from'], From=From, Subject=Subject, Body=Body)
@@ -93,6 +145,7 @@ class Relay(object):
         self.deliver(msg)
 
 
+
 class SMTPReceiver(smtpd.SMTPServer):
     """Receives emails and hands it to the Router for further processing."""
 
@@ -126,12 +179,16 @@ class SMTPReceiver(smtpd.SMTPServer):
         """
 
         try:
-            logging.debug("Message received from Peer: %r, From: %r, to To %r." %
-                          (Peer, From, To))
+            logging.debug("Message received from Peer: %r, From: %r, to To %r." % (Peer, From, To))
             routing.Router.deliver(mail.MailRequest(Peer, From, To, Data))
+        except SMTPError, err:
+            # looks like they want to return an error, so send it out
+            return str(err)
+            undeliverable_message(Data, "Handler raised SMTPError on purpose: %s" % err)
         except:
             logging.exception("Exception while processing message from Peer: %r, From: %r, to To %r." %
                           (Peer, From, To))
+            undeliverable_message(Data, "Error in message %r:%r:%r, look in logs." % (Peer, From, To))
 
 
     def close(self):
@@ -146,12 +203,13 @@ class QueueReceiver(object):
     same way otherwise.
     """
 
-    def __init__(self, queue_dir, sleep=10):
+    def __init__(self, queue_dir, sleep=10, size_limit=0, oversize_dir=None):
         """
         The router should be fully configured and ready to work, the
         queue_dir can be a fully qualified path or relative.
         """
-        self.queue = queue.Queue(queue_dir)
+        self.queue = queue.Queue(queue_dir, pop_limit=size_limit,
+                                 oversize_dir=oversize_dir)
         self.queue_dir = queue_dir
         self.sleep = sleep
 
@@ -168,13 +226,20 @@ class QueueReceiver(object):
                      (self.queue_dir))
         logging.debug("Sleeping for %d seconds..." % self.sleep)
 
+        inq = queue.Queue(self.queue_dir)
+
         while True:
-            inq = queue.Queue(self.queue_dir)
-            key, msg = inq.pop()
-            while key:
-                logging.debug("Pulled message with key: %r off", key)
-                self.process_message(msg)
-                key, msg = inq.pop()
+            keys = inq.keys()
+
+            for key in keys:
+                msg = inq.get(key)
+
+                if msg:
+                    logging.debug("Pulled message with key: %r off", key)
+                    self.process_message(msg)
+                    logging.debug("Removed %r key from queue.", key)
+
+	        inq.remove(key)
 
             if one_shot: 
                 return
@@ -186,15 +251,24 @@ class QueueReceiver(object):
         Exactly the same as SMTPReceiver.process_message but just designed for the queue's
         quirks.
         """
-        Data = str(msg)
-        Peer = self.queue_dir # this is probably harmless but I should check it
-        From = msg['from']
-        To = [msg['to']]
 
         try:
+            Peer = self.queue_dir # this is probably harmless but I should check it
+            From = msg['from']
+            To = [msg['to']]
+
             logging.debug("Message received from Peer: %r, From: %r, to To %r." % (Peer, From, To))
-            routing.Router.deliver(mail.MailRequest(Peer, From, To, Data))
+            routing.Router.deliver(msg)
+        except SMTPError, err:
+            # looks like they want to return an error, so send it out
+            logging.exception("Raising SMTPError when running in a QueueReceiver is unsupported.")
+            undeliverable_message(msg.original, err.message)
         except:
-            logging.exception("Exception while processing message from Peer: %r, From: %r, to To %r." %
-                          (Peer, From, To))
+            logging.exception("Exception while processing message from Peer: "
+                              "%r, From: %r, to To %r." % (Peer, From, To))
+            undeliverable_message(msg.original, "Router failed to catch exception.")
+
+
+
+
 
