@@ -9,8 +9,14 @@ from lamson import server, routing
 import sys, os
 import logging
 import daemon
-from daemon import pidlockfile
+
+try:
+    from daemon import pidlockfile 
+except:
+    from lockfile import pidlockfile 
+
 import imp
+import signal
 
 
 def import_settings(boot_also, from_dir=None, boot_module="config.boot"):
@@ -35,11 +41,11 @@ def daemonize(pid, chdir, chroot, umask, files_preserve=None, do_open=True):
     """
     context = daemon.DaemonContext()
     context.pidfile = pidlockfile.PIDLockFile(pid)
-    context.stdout = open("logs/lamson.out","a+")
-    context.stderr = open("logs/lamson.err","a+")
+    context.stdout = open(os.path.join(chdir, "logs/lamson.out"),"a+")                                                                                                       
+    context.stderr = open(os.path.join(chdir, "logs/lamson.err"),"a+")                                                                                                       
     context.files_preserve = files_preserve or []
     context.working_directory = os.path.expanduser(chdir)
-    
+
     if chroot: 
         context.chroot_directory = os.path.expanduser(chroot)
     if umask != False:
@@ -61,24 +67,6 @@ def drop_priv(uid, gid):
     logging.debug("Now running as uid=%d, gid=%d", os.getgid(), os.getuid())
 
 
-def configure_database(settings, also_create=False):
-    """Configures the lamson database, and can create it as well."""
-    if settings:
-        database = server.Database(settings["metadata"],
-                                  settings["url"],
-                                  log_level=settings["log_level"])
-        database.configure()
-        if also_create:
-            logging.info("Creating the database with %s." % database.url)
-            database.create()
-    else:
-        database = None
-        logging.warning("No database configured, create a database variable in settings.py")
-
-    return database
-
-
-
 
 def make_fake_settings(host, port):
     """
@@ -91,7 +79,6 @@ def make_fake_settings(host, port):
     settings = imp.new_module('settings')
     settings.receiver = server.SMTPReceiver(host, port)
     settings.relay = None
-    settings.database = None
     logging.info("Logging mode enabled, will not send email to anyone, just log.")
 
     return settings
